#ifndef __REQUEST_FLOW_H__599cfdb0
#define __REQUEST_FLOW_H__599cfdb0

#include "TransactionLogger.h"
#include <assert.h>
#include <string>
#include <stdio.h>
#include "picohttpparser.h"

class TcpFlow; // forward declaration

class RequestFlow {
	enum State {
		STATE_READY,
		STATE_HEADERS,
		STATE_BODY,
		STATE_CHUNKED_BODY,
		STATE_ERROR
	};

	TcpFlow *m_tcpFlow;
	time_t tv_sec;
	suseconds_t tv_usec;
	State m_state;
	size_t m_content_length; // content length detected in headers (only relevant for STATE_BODY)

	struct phr_chunked_decoder m_chunk_decoder;

	// Buffer that collects unprocessed request data
	std::string m_buffer;
	size_t m_lastBufferLength;

	// total bytes processed
	size_t m_total_bytes;

	// Parsed request data
	int minor_version;
	const char *method, *path;
	size_t method_len, path_len, num_headers;
	struct phr_header headers[100];

	void emit_headers();
	void emit_body_start();
	void emit_body_chunk(const char *data, size_t data_len, bool last);
	void emit_request_end();
public:
	RequestFlow(TcpFlow *tcpFlow):m_tcpFlow(tcpFlow), m_state(STATE_READY), m_content_length(-1), m_lastBufferLength(0),
		m_total_bytes(0), minor_version(1), method(NULL), path(NULL), method_len(0), path_len(0), num_headers(0) {}
	// push new data to request flow processor
	int push(const char *buf, size_t buf_len, time_t tv_sec, suseconds_t tv_usec);
	size_t get_total_bytes() const { return m_total_bytes; }
};

#endif // __REQUEST_FLOW_H__599cfdb0
