#ifndef __TCP_FLOW__7ab4895a
#define __TCP_FLOW__7ab4895a

#include "RequestFlow.h"
#include "ResponseFlow.h"
#include "HtsenseUtil.h"
#include <nids.h>
#include <memory>
#include <string>
#include <queue>
#include <time.h>
#include <boost/shared_ptr.hpp>

// Structure maintained per flow
class TcpFlow {
	struct tuple4 tuple;
	// timestamp of first packet of this flow
	time_t conn_est_tv_sec;
	suseconds_t conn_est_tv_usec;
public:
	RequestFlow requestFlow; // Requests flow parser
	ResponseFlow responseFlow; // Responses flow parser
	bool m_logResponses; // whether to include reponses in audit log
	bool m_logResponseBody; // whether to include response body in audit log
	std::queue< boost::shared_ptr<TransactionLogger> > m_auditLogQueue; // Audit log composers queue (queue used to support HTTP pipelining feature)

	TcpFlow(const struct tuple4 &tuple, time_t tv_sec, suseconds_t tv_usec, bool logResponses, bool logResponseBody);
	std::string name() const;
	// Callbacks from RequestFlow parser
	void requestStartedCb(const char *method, int method_len, const char *path, int path_len, int http_minor_version, time_t request_tv_sec, suseconds_t request_tv_usec);
	void requestHeaderCb(const char *name, int name_len, const char *value, int value_len) ;
	void requestEndHeadersCb();
	void requestBodyStartCb();
	void requestBodyChunkCb(const char *data, int data_len);
	void requestEndBodyCb();
	void requestFinishedCb();
	// Callbacks from ResponseFlow parser
	void responseStartedCb(int status, const char *message, int message_len, int http_minor_version, time_t response_tv_sec, suseconds_t response_tv_usec);
	void responseHeaderCb(const char *name, int name_len, const char *value, int value_len) ;
	void responseEndHeadersCb();
	void responseBodyStartCb();
	void responseBodyChunkCb(const char *data, int data_len);
	void responseEndBodyCb();
	void responseFinishedCb();
	// Dump audit log to output
	void dumpAuditLog();
};

#endif // __TCP_FLOW__7ab4895a
